/* ___ _      _ _ _     
  / _ (_)_  _| (_) |__  
 / /_)/ \ \/ / | | '_ \ 
/ ___/| |>  <| | | |_) |
\/    |_/_/\_\_|_|_.__/ */

#ifndef PIXLIB_H
#define PIXLIB_H

#define PIXLIB_VERSION 2021TCC

//#ifdef WIN32
// #include <windows.h>
//#endif
#include <stdio.h>

/* Déclaration des types =====================================================*/

//! u8:unsigned char:8 bits 1 octet
typedef unsigned char u8;
//! u16:unsigned short:16 bits 2 octets
typedef unsigned short u16;
//! u32:unsigned int:32 bits 4 octets
typedef unsigned int u32;

//! PIX:une image:Mémoire à libérer après utilisation avec pix_destroy()
typedef struct
{
	int width;
	int height;
	u8 nchannels; // Nombre de channels
	u8 *pixels; // Tableau de pixels (pixels buffer) Liste de valeurs RGB chacune de 0 à 255
} PIX;

//! RGBpixel:un pixel RGB:structure {r,g,b}
typedef struct
{
	u8 r;
	u8 g;
	u8 b;
} RGBpixel, RGBcolor;

//! PIXFONT:Police de caractère 1 channel pour écrire du texte sur une image voir font.c:à libérer après utilisation
typedef struct
{
	u32 width;             // largeur bande en pixels
	u32 taille;            // des caractères 8,12,24,36,72,...
	u32 largeur_caractere; // en pixels
	u32 hauteur_caractere; // en pixels
	u8 *pixels;
} PIXFONT;

// pour utiliser lib depuis C++
//extern "C" {

/* Formats de fichier =======================================================*/

	//! pix_load_bmp:Charge un fichier BMP:un PIX* à libérer après utilisation avec pix_destroy()
	PIX *pix_load_bmp (const char *filename);
	//! pix_save_bmp:Écrit un fichier BMP:int
	int pix_save_bmp (PIX *bmp, const char *filename);

	//! pix_load_jpg:Charge un fichier JPG:un PIX* à libérer après utilisation avec pix_destroy()
	PIX *pix_load_jpg (const char *filepath);
	//! pix_save_jpg:Écrit un fichier JPG:int
	int pix_save_jpg (PIX *img, const char *filename, int quality);

	//! pix_load_png:Charge un fichier PNG:un PIX* à libérer après utilisation avec pix_destroy()
	PIX *pix_load_png (const char* filename);
	//! pix_save_png:Écrit un fichier PNG:int
	int pix_save_png (PIX *img, const char *filename);

	//! pix_load_raw:Charge un fichier RAW:un PIX* à libérer après utilisation avec pix_destroy()
	PIX *pix_load_raw (const char *filename, u8 nchannels);
	//! pix_save_raw:Écrit un fichier RAW:int
	int pix_save_raw (PIX *img, const char *filename, u8 nchannels);

	//! pix_load_tga:Charge un fichier TGA:un PIX* à libérer après utilisation avec pix_destroy()
	PIX* pix_load_tga(const char *filename);
	//! pix_save_tga:Écrit un fichier TGA:void
	void pix_save_tga(PIX* img, PIX* msk, char* filename);

	//PIX* pix_load_webp (const char *filename);
	//! pix_save_webp:Écrit un fichier WEBP:void
	void pix_save_webp (PIX *img, const char *filename);

	//! pix_load_pgm:Charge un fichier PGM:un PIX* à libérer après utilisation avec pix_destroy()
	PIX *pix_load_pgm (const char *filename);
	//! pix_save_pgm:Écrit un fichier PGM:int 0 si erreur, 1 si réussi
	int pix_save_pgm (PIX *img, const char *filename);

/* Operations de base =======================================================*/

	//! pix_create:Créer un PIX aux dimensions x,y pixels et n channels.:PIX* Pointer to data must be freed after use, see pix_destroy().
	PIX *pix_create (int x, int y, u8 nchannels);

	//! pix_copy:Dupliquer un PIX.:PIX* La mémoire doit être libérée après utilisation
	PIX *pix_copy (PIX *img);

	//! pix_destroy:Libérer la mémoire allouée au PIX
	void pix_destroy (PIX *bmp);

/* Operations sur la disposition des pixels =================================*/

	//! pix_flip_vertical:Flip vertical:void
	void pix_flip_vertical (PIX *img);
	//! pix_flip_horizontal:Flip horizontal:void
	void pix_flip_horizontal (PIX *img);
	//! pix_flip_horizontal_vertical:Flip horizontal et vertical:void
	void pix_flip_horizontal_vertical(PIX *img);

	//! pix_redim:Redimensionner un PIX par pourcentage:void
	void pix_redim (PIX *img, unsigned int pourcentX, unsigned int pourcentY);
	//! pix_redimension:Redimensionner un PIX par dimension finale:void
	void pix_redimension (PIX *img, unsigned short int width, unsigned short int height);

	//! pix_extract:Crée un PIX en extrayant un rectangle d'une image.:PIX* Pointer to data must be freed after use, see pix_destroy().
	PIX *pix_extract (PIX *img, int x1, int y1, int x2, int y2);

	//! pix_tile:Découper une image en n = "divisions" images et les écrire sur le disque. "divisions" au carré images seront écrites dans le dossier folder. leur nom est leur index : 0.0.bmp, 0.1.bmp, ...:void
	void pix_tile (PIX* img, short unsigned int divisions, const char* folder);

/* Opérations sur la couleur des pixels =====================================*/

	//! pix_setPixel:Définir les valeurs RGB d'un pixel en x,y:void
	void pix_setPixel (PIX *img, int x, int y, u8 r, u8 g, u8 b);

	//! pix_getPixel:Obtenir les valeurs RGB d'un pixel en x,y:void
	void pix_getPixel (PIX *img, int x, int y, RGBpixel *color);
	//! getPixel:Obtenir les valeurs RGB d'un pixel en x,y:RGBPixel
	RGBpixel getPixel(PIX *img, int x, int y);
	//! getRed:Obtenir la valeur du canal rouge en x,y:u8
	u8 getRed (PIX *img, int x, int y);
	//! getGreen:Obtenir la valeur du canal vert en x,y:u8
	u8 getGreen (PIX *img, int x, int y);
	//! getBlue:Obtenir la valeur du canal bleu en x,y:u8
	u8 getBlue (PIX *img, int x, int y);

	//! pix_fill:Remplir un PIX d'une couleur unie:void
	void pix_fill (PIX *img, RGBcolor *c);

	//! pix_grayscale:Transforme une image en noir et blanc:void
	void pix_grayscale (PIX *img);

	//! pix_invert_colors:Inverser les couleurs:void
	void pix_invert_colors (PIX *img);

	//! pix_eclaircir:Éclaircir ou assombrir une image d'un facteur de i. Cette valeur est ajoutée aux composantes RGB en évitant les dépassements. Ex. 200 + (i=100) est ramené à 255; et 100 - (i=-200) est ramené à 0.:void
	void pix_eclaircir (PIX *img, int i);

	//! pix_color_filter:Appliquer un filtre de couleur:void
	void pix_color_filter (PIX *img, RGBcolor *c);

	//! pix_sepia:Effet Sepia (genre de noir et blanc mais marron):void
	void pix_sepia (PIX *img);

	//! pix_blur:flou:void
	void pix_blur (PIX *img);

	//! pix_noise:noise:void
	void pix_noise (PIX *bmp, int strength);

	//! pix_swap_RGB_vers_BGR:RGB vers BGR:void
	void pix_swap_RGB_vers_BGR(PIX *img);

	//! pix_difference:Remplit un PIX contenant les différences entre deux images:void
	long pix_difference (PIX *img1, PIX *img2, PIX *final);

	//! pix_pixeliser:Pixéliser:void
	void pix_pixeliser (PIX *img, u8 factor);

/* Dessiner des formes =====================================================*/

	//! pix_horizontal_line:Tracer une ligne horizontale. x2 est obligatoire?:void
	void pix_horizontal_line (PIX* img, int x1, int y1, int x2, RGBcolor *c);
	//! pix_vertical_line:Tracer une ligne verticale. y2 est obligatoire?:void
	void pix_vertical_line (PIX* img, int x1, int y1, int y2, RGBcolor *c);
	//! pix_draw_line:Tracer une ligne:void
	void pix_draw_line (PIX* img, int x1, int y1, int x2, int y2, RGBcolor *c);
	//! pix_circle:Tracer un cercle:void
	void pix_circle (PIX* img, int cx, int cy, int radius, RGBcolor* color);
	//! pix_rectangle:Tracer un rectangle:void
	void pix_rectangle (PIX* img, int x1, int x2, int y1, int y2, RGBcolor *c);
	//! pix_square:Tracer un carré:void
	void pix_square (PIX* img, int x1, int y1, int length, RGBcolor *c);

/* Font et écriture de texte sur PIX ======================================*/

	//! pix_load_font:Charger une police depuis le disque:PIXFONT* à libérer après utilisation
	PIXFONT *pix_load_font (char *fontname);

	//! pix_draw_text:Écrire du texte en x,y sur un PIX à la couleur et au font désiré:void
	void pix_draw_text (PIX *img, int x1, int y1, char *text, RGBcolor forecolor, PIXFONT *font);

	//! pix_draw_text_alpha:Écrire du texte avec arrière-plan coloré, transluicide ou opaque:void
	void pix_draw_text_alpha (PIX *img, int x1, int y1, char *text, RGBcolor forecolor, PIXFONT *font, RGBcolor backcolor, float alpha);

	//! pix_save_font:Enregistrer une police:void
	void pix_save_font     (PIXFONT *font, char *filename);
	//! pix_destroy_font:Libérer la mémoire occupée par une police:void
	void pix_destroy_font  (PIXFONT *font);

/* Blitting =============================================================*/

	//! pix_blit:Place un logo sur une image:void
	void pix_blit (PIX *img, int x, int y, PIX *logo);
  
	//! pix_blit_transluicid:Place un logo transluicide sur une image:void
	void pix_blit_transluicid (PIX *img, int x, int y, PIX *logo, float alpha);

	//! pix_blit_transparent:Place un logo transluicide et transparent sur une image:void
	void pix_blit_transparent (PIX *img, int x, int y, PIX *logo, float alpha, RGBpixel *transpaColor);

	//! pix_AlphaChannelByColor:Retourne un PIX 1 channel : le canal alpha d'une image dont une couleur sera transparente au final:PIX* à libérer après utilisation
	PIX* pix_AlphaChannelByColor (PIX* img, RGBcolor* color);

/* Opérations pour la 3D et heightmaps =================================*/
  
	//! pix_blend:Blending, Splatting. Fusionne deux images selon un masque en niveaux de gris:void
	void pix_blend (PIX *img1, PIX *msk, PIX *img2);

	//! pix_blending4:Blending de 4 images avec un masque:PIX* à libérer après utilisation
	//PIX* pix_blending4(PIX* msk, char* ftx1, char* ftx2, char* ftx3, char* ftx4);
	PIX* pix_blending4 (PIX* msk, const char* ftx1, const char* ftx2, const char* ftx3, const char* ftx4);

	//! pix_bake:Bake a (Ambient Occlusion or Lightmap) on b. b is modified after call.:void
	void pix_bake (PIX *a, PIX *b);

	//! pix_toNormalMap:Normal map, un must pour le 3D temps réel:void
	void pix_toNormalMap (PIX* img, int factor);

	//! pix_circlesAlgorithm:Genese de heightmaps:void
	void pix_circlesAlgorithm (PIX* img, int iterations, int radius, int disp, int ax, int ay);

	//! pix_add:Superpose two heightmaps. Average pixel if not 0.:void b is modified after call
	void pix_add (PIX* a, PIX* b);

	//! pix_lightmap:Lightmap:PIX* à libérer après utilisation
	PIX* pix_lightmap (PIX* img, int ambient, float angle_rayons, float hauteur_max_rayons, float hauteur_max_heightmap);

//}
#endif
