#include <pixlib.h>

// 4 façons d'afficher les valeurs RGB des pixels d'une petite image 2x2 pixels
//		rouge, noir
//		blanc, bleu

RGBpixel getPixel(PIX *img, int x, int y)
{
	u8 *p = img->pixels + (x + y * img->width) * img->nchannels;
	RGBpixel pixel = {p[0],p[1],p[2]};
	return pixel;
}

int main()
{
	PIX *img = pix_load_png("img/2X2pixels.png");
	//PIX *img = pix_load_bmp("img/4pixels.bmp"); marche aussi
	if (!img) return 1;

	// affiche ses caractéristiques
	printf("Dimensions:%dX%d pixels, %d channels\n", img->width, img->height, img->nchannels);

	// parcourt les pixels pour afficher leur valeur RGB avec getPixel()
	for(int y = img->height-1; y > -1; y--) // l'image est stockée verticalement à l'envers
	{
		for(int x=0; x < img->width; x++)
		{
			//RGBpixel p;
			//pix_getPixel(img, x,y, &p);
			RGBpixel p = getPixel(img, x,y);
			printf("\t%d-%d-%d\n", p.r, p.g, p.b);
		}
	}

	printf("Acceder directement aux pixels, c'est ce que fait getPixel()\n");
	for(int y = img->height-1; y > -1; y--)
	{
		for(int x=0; x < img->width; x++)
		{
			u8 *p = img->pixels + (x + y * img->width) * img->nchannels;
			printf("\t%d-%d-%d\n", p[0], p[1], p[2]);
		}
	}

	printf("Autre syntaxe img->pixels[a+0]\n");
	for(int y = img->height-1; y > -1; y--)
	{
		for(int x=0; x < img->width; x++)
		{
			int a = (x + y * img->width) * img->nchannels;
			printf("\t%d-%d-%d\n", img->pixels[a+0], img->pixels[a+1], img->pixels[a+2]);
		}
	}

	printf("Traitement avec une seule boucle sans prendre en compte l'ordre d'affichage\n");
	int nb = img->width * img->height;
	for(int n=0; n<nb; n++){
		u8 *p = img->pixels + n*3;
		printf("\t%d-%d-%d\n", p[0], p[1], p[2]);
	}

	// Libère la mémoire. Le gestionnaire de processus efface automatiquement toutes
	// les données de l'exe, mais c'est une bonne habitude de libérer la mémoire
	// dès qu'une portion n'est plus requise, surtout avec de grandes images.
	pix_destroy(img);
	return 0;
}

